<?php

namespace SM\Service;

use SM\Factory\AgendamentoRecursosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class AgendamentoRecursosService
{
    protected $entityManager;
    protected $agendamentoRecursosFactory;
    protected $environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, AgendamentoRecursosFactory $agendamentoRecursosFactory, Environment $environment, NewCode $newCode)
    {
        $this->agendamentoRecursosFactory   = $agendamentoRecursosFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
    }

    public function getAgendamentoRecursoCont()
    {
        return $this->agendamentoRecursosFactory->getAgendamentoRecursoCont();
    } 

    public function saveAgendRecursos($parametros)
    {
        $this->connection->beginTransaction();
        $nrorg = $this->environment->getNrorg();
        $cdoperador = $this->environment->getCdOperador();

        try {
            if ($parametros["NRAGENDRECURSOCONT"] == '-1') {
                $nragendrecursocont = $this->newCode->getCodeByTableName("FCT_AGENDRECURSOCONT", $nrorg, 1, 10, true);

                $dataParams = array(
                    "NRAGENDRECURSOCONT"    => $nragendrecursocont,
                    "NRSEQCONTRATO"         => $parametros["NRSEQCONTRATO"],
                    "NRESTRUTURAM"          => $parametros["NRESTRUTURAM"],
                    "DTINIAGENDAMENTO"      => $parametros["DTINIAGENDAMENTO"],
                    "DTFIMAGENDAMENTO"      => $parametros["DTFIMAGENDAMENTO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => 'S',
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $cdoperador,
                );

                $this->agendamentoRecursosFactory->insertAgendamentoRecursoCont($dataParams);

            } else {
                $dataParams = array(
                    "NRAGENDRECURSOCONT"    => $parametros["NRAGENDRECURSOCONT"],
                    "NRSEQCONTRATO"         => $parametros["NRSEQCONTRATO"],
                    "NRESTRUTURAM"          => $parametros["NRESTRUTURAM"],
                    "DTINIAGENDAMENTO"      => $parametros["DTINIAGENDAMENTO"],
                    "DTFIMAGENDAMENTO"      => $parametros["DTFIMAGENDAMENTO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => 'S',
                    "NRORGULTATU"           => $nrorg,
                    "CDOPERULTATU"          => $cdoperador,
                );

                $this->agendamentoRecursosFactory->updateAgendamentoRecursoCont($dataParams);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteAgendRecursos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->agendamentoRecursosFactory->deleteAgendRecursos(FindService::formatarArray2($parametros['NRAGENDRECURSOCONT']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getRelacEstruturaCont($param)
    {
        $dataParam = array(
            "NRSEQCONTRATO" => isset($param["NRSEQCONTRATO"])? $param["NRSEQCONTRATO"] : null,
        );
        return $this->agendamentoRecursosFactory->getRelacEstruturaCont($dataParam);
    } 

    public function getContratosAgendRecurso()
    {
        return $this->agendamentoRecursosFactory->getContratosAgendRecurso();
    }

    public function getRecursosAgendCont($param)
    {
        $dataParam = array(
            "NRAGENDRECURSOCONT" => $param["NRAGENDRECURSOCONT"]
        );
        return $this->agendamentoRecursosFactory->getRecursosAgendCont($dataParam);
    }
    
    public function getItemContAgendRecurso($param)
    {
        $nrorg = $this->environment->getNrorg();
        
        $dataParam = array(
            "NRSEQCONTRATO" => isset($param["NRSEQCONTRATO"])? $param["NRSEQCONTRATO"]: null,
            "NRORG"         => $nrorg
        );
        return $this->agendamentoRecursosFactory->getItemContAgendRecurso($dataParam);
    } 

    public function getRecursoAgendCont($param)
    {
        $nrorg = $this->environment->getNrorg();
        
        $dataParam = array(
            "NRITEMOCONTRATO"   => isset($param["NRITEMOCONTRATO"])? $param["NRITEMOCONTRATO"]: null,
            "NRORG"             => $nrorg
        );

        return $this->agendamentoRecursosFactory->getRecursoAgendCont($dataParam);
    }
}