<?php

namespace SM\Service;

use SM\Factory\DesalocarRecursosFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class DesalocarRecursosService
{
    protected $entityManager;
    protected $desalocarRecursosFactory;
    protected $environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, DesalocarRecursosFactory $desalocarRecursosFactory, Environment $environment, NewCode $newCode)
    {
        $this->desalocarRecursosFactory     = $desalocarRecursosFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
    }

    public function saveDesalocarRecursos($parametros) {
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();
        $situacao = '2'; // Recurso Desalocado

        $this->connection->beginTransaction();

        try {
            foreach ($parametros["RECURSOS"] as $recurso) {

                if ($recurso["IDSITUARECGEND"] == '1') { // Recurso Alocado.

                    $params = array(
                        "NRDEVAGENDRECURCON"  => $this->newCode->getCodeByTableName("FCT_DEVAGENDRECURCON", $nrorg, 1, 10, true),
                        "NRRECURSOSAGENDCONT" => $recurso["NRRECURSOSAGENDCONT"],
                        "DTDEVAGENDRECURCON"  => $parametros["DTDEVAGENDRECURCON"],
                        "NRORG"               => $nrorg,
                        "IDATIVO"             => "S",
                        "NRORGINCLUSAO"       => $nrorg,
                        "CDOPERINCLUSAO"      => $operador,
                        "DTULTATU"            => null,
                        "NRORGULTATU"         => null,
                        "CDOPERULTATU"        => null  
                    );
                    
                    $this->desalocarRecursosFactory->saveDesalocarRecursos($params);
                    $this->atualizarSituacaoDoAgendamneto($recurso["NRRECURSOSAGENDCONT"], $situacao, $nrorg);
                }
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    /**
    * Função que atualiza o campo situação do agendamento de recuso na tabela de recurso 
    */
    
    public function atualizarSituacaoDoAgendamneto($nrrecursosagendcont,$situacao,$nrorg){
        
        $params = array(
            "IDSITUARECGEND" => $situacao,
            "NRRECURSOSAGENDCONT" => $nrrecursosagendcont,
            "NRORG" => $nrorg
            
        );
        $this->desalocarRecursosFactory->AtualizarSituacaoDoAgendamneto($params);
    }
}