<?php

namespace SM\Service;

use SM\Factory\EquipeTrabalhoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class EquipeTrabalhoService
{
    protected $entityManager;
    protected $equipeTrabalhoFactory;
    protected $environment;
    protected $newCode;
    protected $connection;

    public function __construct(EntityManager $entityManager, EquipeTrabalhoFactory $equipeTrabalhoFactory, Environment $environment, NewCode $newCode)
    {
        $this->equipeTrabalhoFactory        = $equipeTrabalhoFactory;
        $this->connection                   = $entityManager->getConnection();
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
    }

    public function getFctViewRecursos($parametros) {
        
        $dataParam = array( 
            "IDREPETDOMINGO"        => 'N',
            "IDREPETSEGUNDA"        => 'N',
            "IDREPETTERCA"          => 'N',
            "IDREPETQUARTA"         => 'N',
            "IDREPETQUINTA"         => 'N',
            "IDREPETSEXTA"          => 'N',
            "IDREPETSABADO"         => 'N',
            "DTINIAGENDRECURSO"     => isset($parametros["DTINIAGENDRECURSO"])? $parametros["DTINIAGENDRECURSO"] : null,
            "DTFIMAGENDRECURSO"     => isset($parametros["DTFIMAGENDRECURSO"])? $parametros["DTFIMAGENDRECURSO"] : null,
        );
        
        foreach ($parametros["DIASDASEMANA"] as $dia) {
            switch ($dia) {
                case 'IDREPETDOMINGO':
                    $dataParam["IDREPETDOMINGO"] = 'S';
                    break;
                case 'IDREPETSEGUNDA':
                    $dataParam["IDREPETSEGUNDA"] = 'S';
                    break;
                case 'IDREPETTERCA':
                    $dataParam["IDREPETTERCA"] = 'S';
                    break;
                case 'IDREPETQUARTA':
                    $dataParam["IDREPETQUARTA"] = 'S';
                    break;
                case 'IDREPETQUINTA':
                    $dataParam["IDREPETQUINTA"] = 'S';
                    break;
                case 'IDREPETSEXTA':
                    $dataParam["IDREPETSEXTA"] = 'S';
                    break;
                case 'IDREPETSABADO':
                    $dataParam["IDREPETSABADO"] = 'S';
                    break;
                default:
                    # code...
                    break;
            }
        }
        
        return $this->equipeTrabalhoFactory->getFctViewRecursos($dataParam);
    } 
    
    public function updateRecursoEquipeTrabalho($parametros) {
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();
        
        try {

            $dataParams = array(
                "NRRECURSOSAGENDCONT"   => $parametros["NRRECURSOSAGENDCONT"],
                "HRPRIMEIRAENTRADA"     => $parametros["HRPRIMEIRAENTRADA"],
                "HRPRIMEIRASAIDA"       => $parametros["HRPRIMEIRASAIDA"],
                "HRSEGUNDAENTRADA"      => $parametros["HRSEGUNDAENTRADA"],
                "HRSEGUNDASAIDA"        => $parametros["HRSEGUNDASAIDA"],
                "NRTEMPOTOLERANCIA"     => $parametros["NRTEMPOTOLERANCIA"],
                "IDHORAEXTRA"           => $parametros["IDHORAEXTRA"],
                "IDDESCANSOREMU"        => $parametros["IDDESCANSOREMU"],
                "IDESCALATRAB"          => $parametros["IDESCALATRAB"],
                "HRJORNADADIARIA"       => $parametros["HRJORNADADIARIA"],
                "HRINTERVALO"           => $parametros["HRINTERVALO"],
                "HRDESCANSO"            => $parametros["HRDESCANSO"],
                "NRORG"                 => $nrorg,
                "IDATIVO"               => 'S',
                "NRORGULTATU"           => $nrorg,
                "CDOPERULTATU"          => $operador
            );

            $this->equipeTrabalhoFactory->updateRecursoEquipeTrabalho($dataParams);

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    private function verificaDiaSemana($diasDaSemana, $dia) {
        foreach ($diasDaSemana as $value) {
           if($value == $dia) {
            return 'S';
           }
        }
        return 'N';
    }

    /**
     * Valida se um período de alocação está dentro do outro
    **/
    public function validarPeriodosAlocacao($row, $nrorg) {
        $params = array(
            "NRORG"              => $nrorg,
            "NRAGENDRECURSOCONT" => $row["NRAGENDRECURSOCONT"],
            "NRITEMOCONTRATO"    => $row["NRITEMOCONTRATO"],
            "DTINIAGENDRECURSO"  => $row["DTINIAGENDRECURSO"],
            "DTFIMAGENDRECURSO"  => $row["DTFIMAGENDRECURSO"]
        );
        return $this->equipeTrabalhoFactory->validarPeriodosAlocacao($params);
    }

    /**
    * Valida se o item definido já possue alocação para todos 
    * os dias definidos.
    **/
    private function validarDiasAlocacaoItens($row, $nrorg) {
        $quantidade   = count($row);
        $diasDaSemana = array("IDREPETSEGUNDA", "IDREPETTERCA", "IDREPETQUARTA", "IDREPETQUINTA", "IDREPETSEXTA", "IDREPETSABADO", "IDREPETDOMINGO");
        $diasAlocados = $this->validarPeriodosAlocacao($row, $nrorg);
        if(count($diasAlocados) > 0) {
            foreach($diasAlocados as $diaAlocado) {
                $diferenca = ($diaAlocado["QTITEMCONTRATO"] - $diaAlocado["QTALOCADO"]);
                
                if($diferenca != 0 && $diferenca < $quantidade) 
                    return "O item selecionado permite somente a alocação de mais " . $diferenca . " recurso" . ($diferenca > 1 ? "s" : "") ." para o período determinado.";
                
                if($diferenca == 0) {
                    foreach($diasDaSemana as $dia) {
                        if(isset($row[$dia]) && $diaAlocado[$dia] == 'S' && $row[$dia] == 'S')
                            return "O item selecionado já possui a quantidade máxima de alocações para o período definido.";
                    }
                }
            }
        }
        
        return null;
    }

    public function insertRecursoEquipeTrabalho($parametros) {
        $nrorg = $this->environment->getNrorg();
        $operador = $this->environment->getCdOperador();

        $this->connection->beginTransaction();
        
        try {

            foreach ($parametros["CHAVE"] as $recurso) {
                $novoCodigo = $this->newCode->getCodeByTableName("FCT_RECURSOSAGENDCONT", $nrorg, 1, 10, true);

                $dataParams = array(
                    "NRRECURSOSAGENDCONT"   => $novoCodigo,
                    "NRAGENDRECURSOCONT"    => $parametros["NRAGENDRECURSOCONT"],
                    "NRITEMOCONTRATO"       => $parametros["NRITEMOCONTRATO"],
                    "DTINIAGENDRECURSO"     => $parametros["DTINIAGENDRECURSO"],
                    "DTFIMAGENDRECURSO"     => $parametros["DTFIMAGENDRECURSO"],
                    "IDREPETDOMINGO"        => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETDOMINGO"),
                    "IDREPETSEGUNDA"        => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETSEGUNDA"), 
                    "IDREPETTERCA"          => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETTERCA"),
                    "IDREPETQUARTA"         => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETQUARTA"),
                    "IDREPETQUINTA"         => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETQUINTA"),
                    "IDREPETSEXTA"          => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETSEXTA"),
                    "IDREPETSABADO"         => $this->verificaDiaSemana($parametros["DIASDASEMANA"], "IDREPETSABADO"),
                    "NRPARCNEPESSOA"        => $recurso["CHAVE"],
                    "IDTPRECURSO"           => $recurso["CDTIPO"],
                    "IDMOTIVOAGENDAMENTO"   => $parametros["IDMOTIVOAGENDAMENTO"],
                    "IDSITUARECGEND"        => $parametros["IDSITUARECGEND"],
                    "HRPRIMEIRAENTRADA"     => $parametros["HRPRIMEIRAENTRADA"],
                    "HRPRIMEIRASAIDA"       => $parametros["HRPRIMEIRASAIDA"],
                    "HRSEGUNDAENTRADA"      => $parametros["HRSEGUNDAENTRADA"],
                    "HRSEGUNDASAIDA"        => $parametros["HRSEGUNDASAIDA"],
                    "NRTEMPOTOLERANCIA"     => $parametros["NRTEMPOTOLERANCIA"],
                    "IDHORAEXTRA"           => $parametros["IDHORAEXTRA"],
                    "IDDESCANSOREMU"        => $parametros["IDDESCANSOREMU"],
                    "IDESCALATRAB"          => $parametros["IDESCALATRAB"],
                    "HRJORNADADIARIA"       => $parametros["HRJORNADADIARIA"],
                    "HRINTERVALO"           => $parametros["HRINTERVALO"],
                    "HRDESCANSO"            => $parametros["HRDESCANSO"],
                    "NRORG"                 => $nrorg,
                    "IDATIVO"               => 'S',
                    "NRORGINCLUSAO"         => $nrorg,
                    "CDOPERINCLUSAO"        => $operador
                );

                $msgError = $this->validarDiasAlocacaoItens($dataParams, $nrorg);

                if($msgError) {
                    throw new \Exception("Operação bloqueada!<br>" . $msgError);
                }
                
                $this->equipeTrabalhoFactory->insertRecursoEquipeTrabalho($dataParams);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteRecursos($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $this->equipeTrabalhoFactory->deleteRecursos(FindService::formatarArray2($parametros['NRRECURSOSAGENDCONT']));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }
}