import { Utilities } from '@/util/Utilities';
import { Http, Metadata } from '@zeedhi/core';
import {
    Modal, ModalService, Tabs,
    TextInput, Date,
    SelectMultiple,
    Select,
    Textarea,
    Form,
    LoadingService,
    DialogService,
} from '@zeedhi/common';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class EquipeTrabalhoController {
    private currentRowAgendamento : any = {};

    private currentRowRecurso : any = {};

    private modal!: Modal;

    public async abrirModalEquipeTrabalho({ row }: any) {
        this.modal = await ModalService.createFromJson('agendamentoRecursos/modal/modalEquipeTrabalho', true);
        this.currentRowRecurso = row;
        await this.modal.show();
        await this.changeTabsEquipeTrabalho();
    }
    
    public async abrirModalAddRecursoEquipeTrabalho() {
        const gridAgendamentoRecursos = Metadata.getInstance<TekGrid>('grid-agendamento-recursos');
        this.modal = await ModalService.createFromJson('agendamentoRecursos/modal/modalAddRecursoEquipeTrabalho', true);
        this.currentRowAgendamento = gridAgendamentoRecursos.datasource.currentRow;
        this.modal.show();
    }

    public fecharModalAddRecursoEquipeTrabalho() {
        this.modal.hide();
        this.modal?.destroy();
    }

    public get getTituloModalEquipeTrabalho() {
        return `${this.currentRowRecurso.NRCONTRATOFCT} > ${this.currentRowRecurso.NMITEMCONTRATO}`;
    }

    public setFilterCampoItemContrato({ component }: any) {
        const nrseqcontrato = this.currentRowAgendamento.NRSEQCONTRATO;
        try {
            component.datasource.setFilter({
                NRSEQCONTRATO: nrseqcontrato,
            });
        } catch (e) {
            Utilities.showErrorMessage(e);
        }
    }

    public async setFilterCampoRecurso({ component }: any) {
        const datainicio = Metadata.getInstance<Date>(`${component.name.split('_').slice(0, 2).join('_')}_DTINIAGENDRECURSO`).value;
        const datafinal = Metadata.getInstance<Date>(`${component.name.split('_').slice(0, 2).join('_')}_DTFIMAGENDRECURSO`).value;
        const diasdasemana = Metadata.getInstance<SelectMultiple>(`${component.name.split('_').slice(0, 2).join('_')}_DIASDASEMANA`).value;
        await component.datasource.setFilter({
            DTINIAGENDRECURSO: datainicio,
            DTFIMAGENDRECURSO: datafinal,
            DIASDASEMANA: diasdasemana,
        });
    }

    public async validarFieldDeDatasEquipeTrabalho({ component }: any) {
        const diasDaSemana = Metadata.getInstance<SelectMultiple>(`${component.name.split('_').slice(0, 2).join('_')}_DIASDASEMANA`);
        const recurso = Metadata.getInstance<Select>(`${component.name.split('_').slice(0, 2).join('_')}_CHAVE`);
        const dataInicioRecurso = Metadata.getInstance<Date>(`${component.name.split('_').slice(0, 2).join('_')}_DTINIAGENDRECURSO`);
        const dataFinalRecurso = Metadata.getInstance<Date>(`${component.name.split('_').slice(0, 2).join('_')}_DTFIMAGENDRECURSO`);

        this.validarData(dataInicioRecurso, dataFinalRecurso);

        recurso.readonly = !(dataInicioRecurso.value && dataFinalRecurso.value && diasDaSemana.selectValue.length > 0);
        
        if (!recurso.readonly) {
            recurso.datasource.setFilter({
                DTINIAGENDRECURSO: dataInicioRecurso.value,
                DTFIMAGENDRECURSO: dataFinalRecurso.value,
                DIASDASEMANA: diasDaSemana.value,
            });
        }
    }

    public validarData(dataInicioRecurso: Date, dataFinalRecurso: Date) {
        const dataIniAgendamento = this.currentRowAgendamento.DTINIAGENDAMENTO;
        const dataFimAgendamento = this.currentRowAgendamento.DTFIMAGENDAMENTO;

        if (dataInicioRecurso.value && dataFinalRecurso.value && !Utilities.validaPeriodoEntreDatas(dataInicioRecurso.value, dataFinalRecurso.value)) {
            Utilities.showDialogMessage('error', 'Operação bloqueada!', 'A data inicial não pode ser maior que a data final do agendamento.');
            dataInicioRecurso.value = '';
            dataFinalRecurso.value = '';
        } else if (dataInicioRecurso.value && dataFinalRecurso.value && (!Utilities.validaPeriodoEntreDatas(dataIniAgendamento, dataInicioRecurso.value) || !Utilities.validaPeriodoEntreDatas(dataFinalRecurso.value, dataFimAgendamento))) {
            Utilities.showDialogMessage('error', 'Operação bloqueada!', `As datas selecionadas estão fora do período disponível para agendamento de recursos. Escolha um novo intervalo entre ${dataIniAgendamento} e ${dataFimAgendamento}.`);
            dataInicioRecurso.value = '';
            dataFinalRecurso.value = '';
        }
    }

    public changeTabsEquipeTrabalho() {
        // const tekGridRecurso = Metadata.getInstance<TekGrid>('multiple-levels-grid-recurso'); --
        const tabs = Metadata.getInstance<Tabs>('tabs-modal-equipeTrabalho');
        const activeTab = tabs.tabs[tabs.activeTab];
        switch (activeTab.name) {
            case 'tab-info-EquipeTrabalho':
                setTimeout(() => {
                    this.carregaDadosDescEquipeTrabalho();
                }, 100);
                break;
            case 'tab_task_schedule':
                break;
            case 'tab_interlocutor':
                break;
            default:
                // Do nothing
                break;
        }
    }

    public onChangeFieldEscalaTrabalho({ component }: any) {
        const formcargahoraria = Metadata.getInstance<Form>(`${component.name.split('_')[0]}_carga-horaria`);
        const horaprimeiraentrada = Metadata.getInstance<TextInput>(`${component.name.split('_')[0]}_${formcargahoraria.name.split('_')[1]}_HRPRIMEIRAENTRADA`);
        const horaprimeirasaida = Metadata.getInstance<TextInput>(`${component.name.split('_')[0]}_${formcargahoraria.name.split('_')[1]}_HRPRIMEIRASAIDA`);
        const horasegundaentrada = Metadata.getInstance<TextInput>(`${component.name.split('_')[0]}_${formcargahoraria.name.split('_')[1]}_HRSEGUNDAENTRADA`);
        const horasegundasaida = Metadata.getInstance<TextInput>(`${component.name.split('_')[0]}_${formcargahoraria.name.split('_')[1]}_HRSEGUNDASAIDA`);
        const nrtempotolerancia = Metadata.getInstance<Select>(`${component.name.split('_')[0]}_${formcargahoraria.name.split('_')[1]}_NRTEMPOTOLERANCIA`);
        const hrjornadadiaria = Metadata.getInstance<TextInput>(`${component.name.split('_').slice(0, 2).join('_')}_HRJORNADADIARIA`);
        const hrintervalo = Metadata.getInstance<Select>(`${component.name.split('_').slice(0, 2).join('_')}_HRINTERVALO`);
        const hrdescanso = Metadata.getInstance<Select>(`${component.name.split('_').slice(0, 2).join('_')}_HRDESCANSO`);

        if (component.value === 'M') {
            horaprimeiraentrada.readonly = false;
            horaprimeiraentrada.addValidation('required', {});
            horaprimeirasaida.readonly = true;
            horasegundaentrada.readonly = true;
            horasegundasaida.readonly = true;
            nrtempotolerancia.readonly = true;
            hrjornadadiaria.readonly = false;
            hrjornadadiaria.addValidation('required', {});
            hrintervalo.readonly = false;
            hrintervalo.addValidation('required', {});
            hrdescanso.readonly = false;
            hrdescanso.addValidation('required', {});
        } else {
            horaprimeiraentrada.readonly = false;
            horaprimeiraentrada.removeValidation('required');
            horaprimeirasaida.readonly = false;
            horaprimeirasaida.removeValidation('required');
            horasegundaentrada.readonly = false;
            horasegundaentrada.removeValidation('required');
            horasegundasaida.readonly = false;
            horasegundasaida.removeValidation('required');
            nrtempotolerancia.readonly = false;
            nrtempotolerancia.removeValidation('required');
            hrjornadadiaria.readonly = true;
            hrjornadadiaria.removeValidation('required');
            hrintervalo.readonly = true;
            hrintervalo.removeValidation('required');
            hrdescanso.readonly = true;
            hrdescanso.removeValidation('required');
        }
    }

    public changeUtilizaTipoMedida() {
        const UTILIZATIPOMEDIDA = Metadata.getInstance<Select>('formDadosProposalModelViewItens-IDMULTMEDVRTOT').value;
        const TIPOMEDIDA = Metadata.getInstance<Select>('formDadosProposalModelViewItens-CDTPMEDIDASERV');
        const QUANTIDADEMEDIDA = Metadata.getInstance<Select>('formDadosProposalModelViewItens-QTMEDIDAORCA');

        if (UTILIZATIPOMEDIDA === 'S') {
            TIPOMEDIDA.readonly = false;
            TIPOMEDIDA.addValidation('required', {});
            QUANTIDADEMEDIDA.readonly = false;
            QUANTIDADEMEDIDA.addValidation('required', {});
        } else {
            TIPOMEDIDA.readonly = true;
            TIPOMEDIDA.removeValidation('required');
            QUANTIDADEMEDIDA.readonly = true;
            QUANTIDADEMEDIDA.removeValidation('required');
        }
    }

    public async carregaDadosDescEquipeTrabalho() {
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_descricao_NRRECURSOSAGENDCONT').value = this.currentRowRecurso.NRRECURSOSAGENDCONT;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_descricao_NRITEMOCONTRATO').value = this.currentRowRecurso.NMITEMCONTRATO;
        Metadata.getInstance<Date>('form-tab-descricao-equipe-trabalho_descricao_DTINIAGENDRECURSO').value = this.currentRowRecurso.DTINIAGENDRECURSO;
        Metadata.getInstance<Date>('form-tab-descricao-equipe-trabalho_descricao_DTFIMAGENDRECURSO').value = this.currentRowRecurso.DTFIMAGENDRECURSO;
        Metadata.getInstance<SelectMultiple>('form-tab-descricao-equipe-trabalho_descricao_DIASDASEMANA').setValue(this.dadosCampoDiasDaSemana());
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_descricao_NMRECURSO').value = this.currentRowRecurso.NMRECURSO;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_descricao_IDMOTIVOAGENDAMENTO').value = this.currentRowRecurso.IDMOTIVOAGENDAMENTO;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_descricao_IDSITUARECGEND').value = this.currentRowRecurso.IDSITUARECGEND;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRPRIMEIRAENTRADA').value = this.currentRowRecurso.HRPRIMEIRAENTRADA;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRPRIMEIRASAIDA').value = this.currentRowRecurso.HRPRIMEIRASAIDA;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRSEGUNDAENTRADA').value = this.currentRowRecurso.HRSEGUNDAENTRADA;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRSEGUNDASAIDA').value = this.currentRowRecurso.HRSEGUNDASAIDA;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_NRTEMPOTOLERANCIA').value = this.currentRowRecurso.NRTEMPOTOLERANCIA;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_carga-horaria_IDHORAEXTRA').value = this.currentRowRecurso.IDHORAEXTRA;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_carga-horaria_IDDESCANSOREMU').value = this.currentRowRecurso.IDDESCANSOREMU;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_escala-trabalho_IDESCALATRAB').value = this.currentRowRecurso.IDESCALATRAB;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRJORNADADIARIA').value = this.currentRowRecurso.HRJORNADADIARIA;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRINTERVALO').value = this.currentRowRecurso.HRINTERVALO;
        Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRDESCANSO').value = this.currentRowRecurso.HRDESCANSO;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_dados-adicionais_IDALTERACAOEQUIPE').value = this.currentRowRecurso.IDALTERACAOEQUIPE;
        Metadata.getInstance<Date>('form-tab-descricao-equipe-trabalho_dados-adicionais_DTULTATU').value = this.currentRowRecurso.DTULTATU;
        Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_dados-adicionais_NMRECURSOALTEQUIPE').value = this.currentRowRecurso.NRRECURSOALTEQUIPE;
        Metadata.getInstance<Textarea>('form-tab-descricao-equipe-trabalho_dados-adicionais_DSJUSTIFICAALTEQUIPE').value = this.currentRowRecurso.DSJUSTIFICAALTEQUIPE;
    }

    public dadosCampoDiasDaSemana(): string[] {
        const diasMap = [
            { key: 'IDREPETDOMINGO', label: 'IDREPETDOMINGO' },
            { key: 'IDREPETSEGUNDA', label: 'IDREPETSEGUNDA' },
            { key: 'IDREPETTERCA', label: 'IDREPETTERCA' },
            { key: 'IDREPETQUARTA', label: 'IDREPETQUARTA' },
            { key: 'IDREPETQUINTA', label: 'IDREPETQUINTA' },
            { key: 'IDREPETSEXTA', label: 'IDREPETSEXTA' },
            { key: 'IDREPETSABADO', label: 'IDREPETSABADO' },
        ];
    
        return diasMap
            .filter((dia) => this.currentRowRecurso[dia.key] === 'S')
            .map((dia) => dia.label);
    }

    public saveEditEquipeTrabalho() {
        const gridRecursos = Metadata.getInstance<TekGrid>('multiple-levels-grid-recurso');
        const formCargaHoraria = Metadata.getInstance<Form>('form-tab-descricao-equipe-trabalho_carga-horaria');
        const formEscalaTrabalho = Metadata.getInstance<Form>('form-tab-descricao-equipe-trabalho_escala-trabalho');

        if (Utilities.validateForm(formCargaHoraria) && Utilities.validateForm(formEscalaTrabalho)) {
            LoadingService.show();

            const params = {
                NRRECURSOSAGENDCONT: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_descricao_NRRECURSOSAGENDCONT').value,
                HRPRIMEIRAENTRADA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRPRIMEIRAENTRADA').value,
                HRPRIMEIRASAIDA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRPRIMEIRASAIDA').value,
                HRSEGUNDAENTRADA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRSEGUNDAENTRADA').value,
                HRSEGUNDASAIDA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_HRSEGUNDASAIDA').value,
                NRTEMPOTOLERANCIA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_carga-horaria_NRTEMPOTOLERANCIA').value,
                IDHORAEXTRA: Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_carga-horaria_IDHORAEXTRA').value,
                IDDESCANSOREMU: Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_carga-horaria_IDDESCANSOREMU').value,
                IDESCALATRAB: Metadata.getInstance<Select>('form-tab-descricao-equipe-trabalho_escala-trabalho_IDESCALATRAB').value,
                HRJORNADADIARIA: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRJORNADADIARIA').value,
                HRINTERVALO: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRINTERVALO').value,
                HRDESCANSO: Metadata.getInstance<TextInput>('form-tab-descricao-equipe-trabalho_escala-trabalho_HRDESCANSO').value,
            };

            Http.post('/updateRecursoEquipeTrabalho', params).then(() => {
                this.fecharModalAddRecursoEquipeTrabalho();
                gridRecursos.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                Utilities.showSuccessMessage();
                LoadingService.hide();
            });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public saveNewEquipeTrabalho() {
        const gridRecursos = Metadata.getInstance<TekGrid>('multiple-levels-grid-recurso');
        const formDescricao = Metadata.getInstance<Form>('form-tab-add-recurso-equipe-trabalho_descricao');
        const formCargaHoraria = Metadata.getInstance<Form>('form-tab-add-recurso-equipe-trabalho_carga-horaria');
        const formEscalaTrabalho = Metadata.getInstance<Form>('form-tab-add-recurso-equipe-trabalho_escala-trabalho');

        if (Utilities.validateForm(formDescricao) && Utilities.validateForm(formCargaHoraria) && Utilities.validateForm(formEscalaTrabalho)) {
            LoadingService.show();

            const params = {
                NRAGENDRECURSOCONT: this.currentRowAgendamento.NRAGENDRECURSOCONT,
                NRITEMOCONTRATO: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_descricao_NRITEMOCONTRATO').value,
                DTINIAGENDRECURSO: Metadata.getInstance<Date>('form-tab-add-recurso-equipe-trabalho_descricao_DTINIAGENDRECURSO').value,
                DTFIMAGENDRECURSO: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_descricao_DTFIMAGENDRECURSO').value,
                DIASDASEMANA: Metadata.getInstance<SelectMultiple>('form-tab-add-recurso-equipe-trabalho_descricao_DIASDASEMANA').value,
                CHAVE: Metadata.getInstance<SelectMultiple>('form-tab-add-recurso-equipe-trabalho_descricao_CHAVE').selectValue,
                IDMOTIVOAGENDAMENTO: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_descricao_IDMOTIVOAGENDAMENTO').value,
                IDSITUARECGEND: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_descricao_IDSITUARECGEND').value,
                HRPRIMEIRAENTRADA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_carga-horaria_HRPRIMEIRAENTRADA').value,
                HRPRIMEIRASAIDA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_carga-horaria_HRPRIMEIRASAIDA').value,
                HRSEGUNDAENTRADA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_carga-horaria_HRSEGUNDAENTRADA').value,
                HRSEGUNDASAIDA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_carga-horaria_HRSEGUNDASAIDA').value,
                NRTEMPOTOLERANCIA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_carga-horaria_NRTEMPOTOLERANCIA').value,
                IDHORAEXTRA: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_carga-horaria_IDHORAEXTRA').value,
                IDDESCANSOREMU: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_carga-horaria_IDDESCANSOREMU').value,
                IDESCALATRAB: Metadata.getInstance<Select>('form-tab-add-recurso-equipe-trabalho_escala-trabalho_IDESCALATRAB').value,
                HRJORNADADIARIA: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_escala-trabalho_HRJORNADADIARIA').value,
                HRINTERVALO: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_escala-trabalho_HRINTERVALO').value,
                HRDESCANSO: Metadata.getInstance<TextInput>('form-tab-add-recurso-equipe-trabalho_escala-trabalho_HRDESCANSO').value,
            };

            Http.post('/insertRecursoEquipeTrabalho', params).then(() => {
                this.fecharModalAddRecursoEquipeTrabalho();
                gridRecursos.datasource.get();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                Utilities.showSuccessMessage();
                LoadingService.hide();
            });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public actionDesalocarRecurso: boolean = true;

    public alterarActionDesalocar({ component }: any) {
        if (component.selectedRows.length > 0) {
            this.actionDesalocarRecurso = false;
        } else {
            this.actionDesalocarRecurso = true;
        }
    }

    public desabilitarActionDesalocar() {
        this.actionDesalocarRecurso = true;
    }
    
    public async abrirModalDesalocarRecurso() {
        this.modal = await ModalService.createFromJson('agendamentoRecursos/modal/modalDesalocarRecursoEquipeTrabalho', true);
        this.modal.show();
    }

    public desalocarRecurso() {
        const gridRecursos = Metadata.getInstance<TekGrid>('multiple-levels-grid-recurso');
        const form = Metadata.getInstance<Form>('form-tab-desalocar-recurso-equipe-trabalho');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            const params = {
                RECURSOS: gridRecursos.selectedRows,
                DTDEVAGENDRECURCON: Metadata.getInstance<Date>('form-tab-desalocar-recurso-equipe-trabalho-DTDEVAGENDRECURCON').value,
            };
            Http.post('/saveDesalocarRecursos', params).then(() => {
                this.fecharModalAddRecursoEquipeTrabalho();
                gridRecursos.datasource.get();
                gridRecursos.selectedRows = [];
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                Utilities.showSuccessMessage();
                LoadingService.hide();
            });
        } else {
            Utilities.showAlertCamposObrigatorios();
        }
    }

    public dialogDeleteRecursos({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'EquipeTrabalhoController.deleteRecursos');
    }

    public async deleteRecursos() {
        const grid = Metadata.getInstance<TekGrid>('multiple-levels-grid-recurso');
        LoadingService.show();

        Http.post('/deleteRecursos', {
            NRRECURSOSAGENDCONT: grid.selectedRows.map((element: any) => element.NRRECURSOSAGENDCONT),
        }).then(() => {
            grid.datasource.get();
            Utilities.showSuccessMessage();
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            DialogService.hide();
        });
    }
}
