import {
    Http, IDictionary, Metadata,
} from '@zeedhi/core';
import {
    ApexChart, Text,
} from '@zeedhi/common';
import {
    IterableComponentRender,
} from '@zeedhi/teknisa-components-common';

export class FrameContractsController {
    public async mountCharts({ component }: any, route: string, date: any, text: string) {
        // const dateRange = Metadata.getInstance<DateRange>('range-dashboard').value;
        const dataLabels: any = [];
        const dataSeries: any = [];
        const dataValue: any = [];
        const periodText = Metadata.getInstance<Text>(text);

        if (date !== '') periodText.text = `5 Principais Escopos | Período: ${date[0]} - ${date[1]}`;
        else periodText.text = '5 Principais Escopos | Período: Últimos 30 dias';

        await Http.post(route, {
            DTINICIO: (date === '' || date.length < 1) ? null : date[0],
            DTFINAL: (date === '' || date.length < 2) ? null : date[1],
        }).then((response: IDictionary) => {
            if (response.data.dataset.chart.length) {
                response.data.dataset.chart.forEach((dados: IDictionary) => {
                    dataLabels.push(dados.NMTIPOORCAMENTO);
                    dataSeries.push(dados.QUANTIDADE);
                    dataValue.push(dados.VALOR);
                    console.warn(dados.VALOR);
                });
            } else {
                dataLabels.push('Nenhum dado disponível');
                dataSeries.push(100);
                dataValue.push(0);
            }
        });
        const options = { series: dataSeries, labels: dataLabels, value: dataValue };
        component.updateChart(options);
    }

    public mountChartAprovedContracts({ component }: any) {
        const cApproved = Metadata.getInstance<ApexChart>('donut-chart-ApprovedContracts_CONTRATO');
        const text = Metadata.getInstance<Text>('subtitle-ApprovedContracts').name;
        const route = '/approvedContracts';

        const date = (component.component === 'ZdDateRange') ? component.value : '';
        this.mountCharts({ component: cApproved }, route, date, text);
    }

    public mountChartTerminatedContracts({ component }: any) {
        const cTerminated = Metadata.getInstance<ApexChart>('donut-chart-terminatedContracts_CONTRATO');
        const text = Metadata.getInstance<Text>('subtitle-terminatedContracts').name;
        const route = '/terminatedContracts';

        const date = (component.component === 'ZdDateRange') ? component.value : '';
        this.mountCharts({ component: cTerminated }, route, date, text);
    }

    public mountChartRenegotiatedContracts({ component }: any) {
        const cRenegotiated = Metadata.getInstance<ApexChart>('donut-chart-renegotiatedContracts_CONTRATO');
        const text = Metadata.getInstance<Text>('subtitle-renegotiatedContracts').name;
        const route = '/renegotiatedContracts';

        const date = (component.component === 'ZdDateRange') ? component.value : '';
        this.mountCharts({ component: cRenegotiated }, route, date, text);
    }

    public setFilterCard({ component }: any) {
        const tekIterable = Metadata.getInstance<IterableComponentRender>('tekCard-contractsPendingBilling');
        const dateComp = (component.component === 'ZdDateRange') ? component.value : '';
        const date1 = (dateComp === '' || dateComp.length < 1) ? 1 : dateComp[0];
        const date2 = (dateComp === '' || dateComp.length < 2) ? 1 : dateComp[1];

        tekIterable.datasource.setFilter({
            DTINI: date1,
            DTFIN: date2,
        });
        tekIterable.datasource.get();
    }
}
