import {
    Modal, ModalService, Select, TextInput, Text, Form, LoadingService, DialogService,
} from '@zeedhi/common';
import { Utilities } from '@/util/Utilities';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class ServicoEscopoTabCompetenciaController {
    private basicModal!: Modal;

    public expandir({ component }: any) {
        const form = Metadata.getInstance<Form>(component.name.split('_')[1]);
        form.isVisible = !form.isVisible;

        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public async abrirModalEdicaoCompetencia({ row }: any) {
        await this.exibirModalCompetencia();
        Metadata.getInstance<Text>('titleText').text = 'Editar Competência';
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRPARAMFATITESCSERV').value = row.NRPARAMFATITESCSERV;
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETINI').value = row.NRDIACOMPETINI;
        Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETINI').value = row.IDMESCOMPETINI;
        Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETFIN').value = row.IDMESCOMPETFIN;
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETFIN').value = row.NRDIACOMPETFIN;
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-QTDIASAPURACAO').value = row.QTDIASAPURACAO;
        Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDTPCALCVENC').value = row.IDTPCALCVENC;
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACALCVENC').value = row.NRDIACALCVENC;
        Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCALCVENC').value = row.IDMESCALCVENC;
        Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-QTDIASFATUVENC').value = row.QTDIASFATUVENC;
    }

    public async abrirModalAdicionaCompetencia() {
        await this.exibirModalCompetencia();
        Metadata.getInstance<Text>('titleText').text = 'Nova Competência';
    }

    public async exibirModalCompetencia() {
        this.basicModal = await ModalService.createFromJson('escopoFaturamento/modal/servicoEscopoModalCompetencia', true);
        this.basicModal.show();
    }

    public fecharModalCompetencia() {
        this.basicModal.hide();
        this.basicModal?.destroy();
    }

    // Os métodos abaixo são relacionados ao Modal de Edição e a adição de Competência para os Parametros de Faturamento.

    public validaCampoVazio(campo : string) : boolean {
        if ((campo === '') || (campo === undefined) || (campo === null)) {
            return true;
        }
        return false;
    }

    public validarPeriodoParamFatCompetencia() {
        const diaInicial = Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETINI').value;
        const diaFinal = Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETFIN').value;
        const mesInicial = Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETINI').value;
        const mesFinal = Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETFIN').value;

        if (!this.validaCampoVazio(diaInicial) && !this.validaCampoVazio(diaFinal)) {
            if (mesInicial === mesFinal && diaInicial > diaFinal) {
                Utilities.alertMessage('Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final.', 'error');
                Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETINI').value = null;
                Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETFIN').value = null;
            } else if (mesInicial > mesFinal) {
                Utilities.alertMessage('Operação bloqueada.<br/>O dia da competência inicial deve anterior que o dia final', 'error');
                Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETINI').value = null;
                Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETFIN').value = null;
            }
        }
    }

    public controlarTipoCalculoVencimento({ component }: any) {
        const idtpcalcvenc = component.value;
        const diaVencimento = Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACALCVENC');
        const mesCalculoVencimento = Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCALCVENC');
        const diasMinimosFatVenc = Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-QTDIASFATUVENC');

        if (idtpcalcvenc === '1') { //  Dia fixo para a data de vencimento
            diaVencimento.label = 'Dia de Vencimento';
            
            mesCalculoVencimento.readonly = false;
            mesCalculoVencimento.addValidation('required', {});
            
            diasMinimosFatVenc.readonly = false;
            diasMinimosFatVenc.addValidation('required', {});
        } else if (idtpcalcvenc === '2') { // Quantidade de dias após a data de emissão
            diaVencimento.label = 'Quantidade de Dias';

            mesCalculoVencimento.readonly = true;
            mesCalculoVencimento.removeValidation('required');
            mesCalculoVencimento.value = '';
            
            diasMinimosFatVenc.readonly = true;
            diasMinimosFatVenc.removeValidation('required');
            mesCalculoVencimento.value = '';
        }
    }

    public saveServEscTabCompetencia() {
        const formServEscComp = Metadata.getInstance<Form>('form_modal_servico_escopo_competencia');
        const formServEscCompDadosVenc = Metadata.getInstance<Form>('form_modal_servico_escopo_competencia_DadosVencimento');
        const gridServEsc = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-servico-escopo');
        const gridServEscComp = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-servicos-escopo-competencia');
        const nritescopoproposta = gridServEsc.datasource.currentRow.NRITESCOPOPROPOSTA;

        const params = {
            NRPARAMFATITESCSERV: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRPARAMFATITESCSERV').value,
            NRITESCOPOPROPOSTA: nritescopoproposta,
            NRDIACOMPETINI: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETINI').value,
            IDMESCOMPETINI: Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETINI').value,
            IDMESCOMPETFIN: Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCOMPETFIN').value,
            NRDIACOMPETFIN: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACOMPETFIN').value,
            QTDIASAPURACAO: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-QTDIASAPURACAO').value,
            IDTPCALCVENC: Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDTPCALCVENC').value,
            NRDIACALCVENC: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-NRDIACALCVENC').value,
            IDMESCALCVENC: Metadata.getInstance<Select>('servicoEscopoCompetenciaModal-IDMESCALCVENC').value,
            QTDIASFATUVENC: Metadata.getInstance<TextInput>('servicoEscopoCompetenciaModal-QTDIASFATUVENC').value,
        };

        if (Utilities.validateForm(formServEscComp) && Utilities.validateForm(formServEscCompDadosVenc)) {
            LoadingService.show();
            Http.post('/saveServEscTabCompetencia', params).then(() => {
                gridServEscComp.datasource.get();
                this.fecharModalCompetencia();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                if (error.response.data.error === 'EXISTE') {
                    Utilities.showDialogMessage('error', 'Operação Bloqueada!', 'Já existe um registro para o período de competência informado.');
                } else {
                    Utilities.showErrorMessage(error);
                }
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public showConfirmDeleteMessageServEscTabCompetencia({ event }: any) {
        event!.preventDefault();
        DialogService.show({
            name: 'buttons-dialog-operador',
            type: 'warning',
            title: 'Excluir Registro',
            text: 'Deseja realmente excluir todos os registros selecionados?',
            buttons: [
                {
                    name: 'NAO',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => {
                            DialogService.hide();
                        },
                    },
                },
                {
                    name: 'SIM',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: {
                        click: () => {
                            this.deleteServEscTabCompetencia();
                            DialogService.hide();
                        },
                    },
                },
            ],
        });
    }

    public deleteServEscTabCompetencia() {
        const grid = Metadata.getInstance<TekGrid>('basic-tek-grid-tab-servicos-escopo-competencia');
        LoadingService.show();
        Http.post('/deleteServEscTabCompetencia', grid.selectedRows)
            .then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
    }
}
